/*
    Robot servo control client

    Sends key data to the servo, to control a microcontroller

    (C) 2004 Jason Hunt

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Contact: huntjas2@msu.edu

    Compile with -lwsock32
*/

#include <stdio.h>
#include <stdlib.h>
#include <winsock.h>
#include <conio.h>


#define VERSION "1.04"
#define debug 0
#define PORT 1200

unsigned int done = 0;


/* return the number of seconds */
unsigned int get_hund( void )
{
    SYSTEMTIME t;

    GetSystemTime(&t);

    return(t.wMilliseconds / 10);
}
/* end of get_seconds */


/* returns true of the passed key is pressed, the program does not need focus */
int key_down(SHORT k)
{
    return(GetAsyncKeyState(k) & 0x8000);
}
/* end of key_down */


/* sends the updated control information to the server */
int send_control_state(SOCKET sockfd, char left, char right, unsigned char servo1, unsigned char servo2)
{
    char p[5];

    p[0] = 0x01; // command byte data (01 == update movement data)
    p[1] = left;
    p[2] = right;
    p[3] = servo1;
    p[4] = servo2;

    if (send(sockfd,p,5,0) == SOCKET_ERROR)
    {
        printf("\nUnable to send data\n");
        return(1);
    }

    return(0);
}
/* end of send_control_state */

/* sends the updated control information to the server */
int send_keep_alive(SOCKET sockfd)
{
    char p[5];

    p[0] = 0x00; // command byte data (01 == update movement data)
    p[1] = 0x00;
    p[2] = 0x00;
    p[3] = 0x00;
    p[4] = 0x00;

    if (send(sockfd,p,5,0) == SOCKET_ERROR)
    {
        printf("\nUnable to send data\n");
        return(1);
    }

    return(0);
}
/* end of send_control_state */


int update_key_values(char &lm, char &rm, unsigned char &p, unsigned char &t)
{
    char old_lm = lm, old_rm = rm;
    unsigned char old_p = p, old_t = t;

    if (key_down(VK_UP))
    {
        if (key_down(VK_LEFT))  {lm = 'S'; rm = 'F'; } else // forward and left
        if (key_down(VK_RIGHT)) {lm = 'F'; rm = 'S'; } else // forward and right
        {
                lm = 'F'; rm = 'F'; // normal forward
        }
    } else
    if (key_down(VK_DOWN))
    {
        if (key_down(VK_LEFT))  {lm = 'S'; rm = 'R'; } else // reverse and left
        if (key_down(VK_RIGHT)) {lm = 'R'; rm = 'S'; } else // reverse and right
        {
            lm = 'R'; rm = 'R'; // normal reverse
        }
    } else
    if (key_down(VK_LEFT)) // standing left
    {
        lm = 'R';
        rm = 'F';
    } else
    if (key_down(VK_RIGHT)) // standing right
    {
        lm = 'F';
        rm = 'R';
    }else
    {
        lm = 'S';
        rm = 'S';
    }

    if (key_down('W')) // camera up
        if (t < 255) t++;

    if (key_down('S')) // camera down
        if (t > 0) t--;

    if (key_down('A')) // camera left
        if (p < 255) p++;

    if (key_down('D')) // camera right
        if (p > 0) p--;

    if (old_p != p || old_t != t) Sleep(50); // delay for camera update

    return(old_lm != lm || old_rm != rm || old_p != p || old_t != t);
}


/* attempt to connect to a server */
int connect_to_server(SOCKET &s, char *hostname)
{
    // create the socket
    s = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

    if (s == INVALID_SOCKET)
    {
        printf("Socket error\n");
        return(0);
    }

    printf("Connecing to %s\n", hostname);

    LPHOSTENT hostent;

    hostent=gethostbyname(hostname);

    if (!hostent)
    {
        fprintf(stderr,"\nHost name error\n");
        return(0);
    }

    /* setup address information */
    SOCKADDR_IN server;

    server.sin_addr=*((LPIN_ADDR)*hostent->h_addr_list);
    server.sin_port=htons(PORT);
    server.sin_family=AF_INET;


    int connected = 0;
    int attempt_count = 0;

    while (!connected && ! key_down(VK_ESCAPE))
    {
        if(connect(s,(LPSOCKADDR)&server, sizeof(struct sockaddr)) == SOCKET_ERROR)
        {
            attempt_count++;
            printf("Connection time out (%d)\n", attempt_count);
        }
        else
            connected = 1;
    }

    if (key_down(VK_ESCAPE))
    {
        done = 1;
        return(0);
    }

    u_long arg = 1;
    if (ioctlsocket(s, FIONBIO, &arg) != 0)
    {
        printf("\nUnable to makesocket non blocking\n");
        return(0);
    }

    return(connected);
}


void connected_loop( SOCKET &server)
{
    // reset the position date
    unsigned char pan = 128, tilt = 128;
    char left_motor = 'S';
    char right_motor = 'S';

    unsigned int cur_hund = get_hund();
    unsigned int hund_count = 0;

    // get user key data and send to server
    while(!done)
    {
        // has the hundred's place changed?
        if (cur_hund != get_hund())
        {
            cur_hund = get_hund(); // set new
            hund_count++;

            // time to update?
            if (hund_count > 40)
            {
                if(send_keep_alive(server)) return;
//                if(send_control_state(server, left_motor, right_motor,  pan, tilt)) return;
                hund_count = 0;
            }
        }

        // client exit
        if (key_down(VK_ESCAPE))
        {
            done = 1;
            return;
        }

        // movement has changed, send new data
        if (update_key_values(left_motor, right_motor, pan, tilt))
        {
            if(send_control_state(server, left_motor, right_motor,  pan, tilt)) return;

            printf("left_motor: %c  right_motor: %c  pan: %3d tilt: %3d\n",
                   left_motor, right_motor, pan, tilt);
        }
    }
}



int main(int argc, char *argv[])
{
    WSADATA wsda;
    WSAStartup(MAKEWORD(1, 1),&wsda);

    int packet_size;

    char hostname[1024];

    printf("Robot control program version %s (C) 2004 Jason Hunt\n\n", VERSION);

    if (debug)
    {
        printf("DEBUG MODE\n");
        strcpy(hostname, "127.0.0.1");
    }
    else
    {
        if (argc != 2)
        {
            printf("Enter hostname: ");
            scanf("%s", hostname);
        }
        else
            strcpy(hostname, argv[1]);
    }

    while (!done)
    {
        /* create the socket */
        SOCKET sockfd;

        while (!connect_to_server(sockfd, hostname) && !done);

        if (done) break;

        printf("\nConnected, waiting for user input\n");

        connected_loop(sockfd);

        closesocket(sockfd);
    }

    printf("\n\n");

    WSACleanup();

    return(0);
}




